@@ -38,7 +38,7 @@ unsigned int SPCtg::ctgWidth = 0;
 /**
   @brief Constructor.  Initializes 'runFlags' to zero for the single-split root.
  */
-SplitPred::SplitPred(const PMTrain *_pmTrain, const RowRank *_rowRank, SamplePred *_samplePred, unsigned int _bagCount) : rowRank(_rowRank), pmTrain(_pmTrain), bagCount(_bagCount), samplePred(_samplePred) {
+SplitPred::SplitPred(const PMTrain *_pmTrain, const RowRank *_rowRank, SamplePred *_samplePred, unsigned int _bagCount) : rowRank(_rowRank), pmTrain(_pmTrain), bagCount(_bagCount), noSet(bagCount * pmTrain->NPredFac()), samplePred(_samplePred) {
 }
 
 
@@ -112,7 +112,7 @@ void SPCtg::DeImmutables() {
    @param samplePred holds (re)staged node contents.
  */
 SPReg::SPReg(const PMTrain *_pmTrain, const RowRank *_rowRank, SamplePred *_samplePred, unsigned int _bagCount) : SplitPred(_pmTrain, _rowRank, _samplePred, _bagCount), ruMono(0) {
-  run = new Run(0, pmTrain->NRow(), _bagCount);
+  run = new Run(0, pmTrain->NRow(), noSet);
 }
 
 
@@ -124,7 +124,7 @@ SPReg::SPReg(const PMTrain *_pmTrain, const RowRank *_rowRank, SamplePred *_samp
    @param sampleCtg is the sample vector for the tree, included for category lookup.
  */
 SPCtg::SPCtg(const PMTrain *_pmTrain, const RowRank *_rowRank, SamplePred *_samplePred, const std::vector<SampleNode> &_sampleCtg, unsigned int _bagCount): SplitPred(_pmTrain, _rowRank, _samplePred, _bagCount), sampleCtg(_sampleCtg) {
-  run = new Run(ctgWidth, pmTrain->NRow(), _bagCount);
+  run = new Run(ctgWidth, pmTrain->NRow(), noSet);
 }
 
 
@@ -142,12 +142,8 @@ void SplitPred::LevelInit(IndexLevel &index) {
   std::vector<bool> unsplitable(levelCount);
   std::fill(unsplitable.begin(), unsplitable.end(), false);
   LevelPreset(index, unsplitable);
-
-  std::vector<unsigned int> safeCount;
-  Splitable(unsplitable, safeCount);
-
   SetPrebias(index); // Depends on state from LevelPreset()
-  RunOffsets(safeCount);
+  Splitable(unsplitable);
 }
 
 
@@ -198,8 +194,8 @@ void SPReg::LevelInit(IndexLevel &index) {
 
    @return void.
  */
-void SPReg::RunOffsets(const std::vector<unsigned int> &safeCount) {
-  run->RunSets(safeCount);
+void SPReg::RunOffsets(const std::vector<unsigned int> &runCount) {
+  run->RunSets(runCount);
   run->OffsetsReg();
 }
 
@@ -207,8 +203,8 @@ void SPReg::RunOffsets(const std::vector<unsigned int> &safeCount) {
 /**
    @brief Sets quick lookup offsets for Run object.
  */
-void SPCtg::RunOffsets(const std::vector<unsigned int> &safeCount) {
-  run->RunSets(safeCount);
+void SPCtg::RunOffsets(const std::vector<unsigned int> &runCount) {
+  run->RunSets(runCount);
   run->OffsetsCtg();
 }
 
@@ -220,7 +216,7 @@ void SPCtg::RunOffsets(const std::vector<unsigned int> &safeCount) {
 
    @return void.
 */
-void SplitPred::Splitable(const std::vector<bool> &unsplitable, std::vector<unsigned int> &safeCount) {
+void SplitPred::Splitable(const std::vector<bool> &unsplitable) {
     // TODO:  Pre-empt overflow by walking wide subtrees depth-first.
   int cellCount = levelCount * nPred;
 
@@ -238,10 +234,10 @@ void SplitPred::Splitable(const std::vector<bool> &unsplitable, std::vector<unsi
       continue; // No predictor splitable
     unsigned int splitOff = levelIdx * nPred;
     if (predFixed == 0) { // Probability of predictor splitable.
-      ScheduleProb(levelIdx, &ruPred[splitOff], safeCount);
+      PrescheduleProb(levelIdx, &ruPred[splitOff]);
     }
     else { // Fixed number of predictors splitable.
-      ScheduleFixed(levelIdx, &ruPred[splitOff], &heap[splitOff], safeCount);
+      PrescheduleFixed(levelIdx, &ruPred[splitOff], &heap[splitOff]);
     }
   }
 
@@ -260,10 +256,10 @@ void SplitPred::Splitable(const std::vector<bool> &unsplitable, std::vector<unsi
 
    @return void, with output vector.
  */
-void SplitPred::ScheduleProb(unsigned int levelIdx, const double ruPred[], std::vector<unsigned int> &safeCount) {
+void SplitPred::PrescheduleProb(unsigned int levelIdx, const double ruPred[]) {
   for (unsigned int predIdx = 0; predIdx < nPred; predIdx++) {
     if (ruPred[predIdx] < predProb[predIdx]) {
-      (void) ScheduleSplit(levelIdx, predIdx, safeCount);
+      (void) Preschedule(levelIdx, predIdx);
     }
   }
 }
@@ -280,7 +276,7 @@ void SplitPred::ScheduleProb(unsigned int levelIdx, const double ruPred[], std::
 
    @return void, with output vector.
  */
-void SplitPred::ScheduleFixed(unsigned int levelIdx, const double ruPred[], BHPair heap[], std::vector<unsigned int> &safeCount) {
+void SplitPred::PrescheduleFixed(unsigned int levelIdx, const double ruPred[], BHPair heap[]) {
   // Inserts negative, weighted probability value:  choose from lowest.
   for (unsigned int predIdx = 0; predIdx < nPred; predIdx++) {
     BHeap::Insert(heap, predIdx, -ruPred[predIdx] * predProb[predIdx]);
@@ -290,25 +286,31 @@ void SplitPred::ScheduleFixed(unsigned int levelIdx, const double ruPred[], BHPa
   unsigned int schedCount = 0;
   for (unsigned int heapSize = nPred; heapSize > 0; heapSize--) {
     unsigned int predIdx = BHeap::SlotPop(heap, heapSize - 1);
-    schedCount += ScheduleSplit(levelIdx, predIdx, safeCount) ? 1 : 0;
+    schedCount += Preschedule(levelIdx, predIdx) ? 1 : 0;
     if (schedCount == predFixed)
       break;
   }
 }
 
 
-bool SplitPred::ScheduleSplit(unsigned int levelIdx, unsigned int predIdx, std::vector<unsigned int> &safeCount) {
-  unsigned int runCount, bufIdx;
-  if (bottom->ScheduleSplit(levelIdx, predIdx, runCount, bufIdx)) {
-    SplitCoord sg;
-    if (runCount > 0) {
-      sg.InitEarly(splitCoord.size(), levelIdx, predIdx, bufIdx, safeCount.size());
-      safeCount.push_back(runCount);
-    }
-    else {
-      sg.InitEarly(splitCoord.size(), levelIdx, predIdx, bufIdx, run->NoRun());
-    }
-    splitCoord.push_back(sg);
+/**
+   @brief Initializes the list of split candidates with extant cells.
+
+   @return true iff candidate has been prescheduled.
+ */
+bool SplitPred::Preschedule(unsigned int levelIdx, unsigned int predIdx) {
+  SplitCoord sg;
+  return sg.Preschedule(bottom, levelIdx, predIdx, splitCoord);
+}
+
+
+bool SplitCoord::Preschedule(Bottom *bottom, unsigned int _levelIdx, unsigned int _predIdx, std::vector<SplitCoord> &splitCoord) {
+  unsigned int _bufIdx;
+  if (bottom->Preschedule(_levelIdx, _predIdx, _bufIdx)) {
+    levelIdx = _levelIdx;
+    predIdx = _predIdx;
+    bufIdx = _bufIdx;
+    splitCoord.push_back(*this);
     return true;
   }
   else {
@@ -317,6 +319,64 @@ bool SplitPred::ScheduleSplit(unsigned int levelIdx, unsigned int predIdx, std::
 }
 
 
+/**
+   @brief Walks the list of split candidates and invalidates those which
+   restaging has marked unsplitable as well as singletons persisting since
+   initialization or as a result of bagging.  Fills in run counts, which
+   values restaging has established precisely.
+*/
+void SplitPred::ScheduleSplits(const IndexLevel &index) {
+  std::vector<unsigned int> runCount;
+  std::vector<SplitCoord> sc2;
+  for (auto & sg : splitCoord) {
+    sg.Schedule(bottom, index, noSet, runCount, sc2);
+  }
+  splitCoord = std::move(sc2);
+
+  RunOffsets(runCount);
+}
+
+
+/**
+   @brief Retains split coordinate iff target is not a singleton.  Pushes
+   back run counts, if applicable.
+
+   @param sg holds partially-initialized split coordinates.
+
+   @param runCount accumulates nontrivial run counts.
+
+   @param sc2 accumulates "actual" splitting coordinates.
+
+   @return void, with output reference vectors.
+ */
+void SplitCoord::Schedule(const Bottom *bottom, const IndexLevel &index, unsigned int noSet, std::vector<unsigned int> &runCount, std::vector<SplitCoord> &sc2) {
+  unsigned int rCount;
+  if (bottom->ScheduleSplit(levelIdx, predIdx, rCount)) {
+    InitLate(bottom, index, sc2.size(), rCount > 1 ? runCount.size() : noSet);
+    if (rCount > 1) {
+      runCount.push_back(rCount);
+    }
+    sc2.push_back(*this);
+  }
+}
+
+
+/**
+   @brief Initializes field values known only following restaging.  Entry
+   singletons should not reach here.
+
+   @return true iff non-singleton.
+ */
+void SplitCoord::InitLate(const Bottom *bottom, const IndexLevel &index, unsigned int _splitPos, unsigned int _setIdx) {
+  splitPos = _splitPos;
+  setIdx = _setIdx;
+  unsigned int extent;
+  preBias = index.SplitFields(levelIdx, idxStart, extent, sCount, sum);
+  implicit = bottom->AdjustDense(levelIdx, predIdx, idxStart, extent);
+  idxEnd = idxStart + extent - 1; // May overflow if singleton:  invalid.
+}
+
+
 /**
    @brief Base method.  Deletes per-level run and split-flags vectors.
 
@@ -459,40 +519,9 @@ int SPReg::MonoMode(unsigned int levelIdx, unsigned int predIdx) const {
 }
 
 
-/**
- @brief Sets those fields known before restaging.
-  isDense set following restage.
- runCount may be reset to unity following restage.
-
- @return void.
-*/
-void SplitCoord::InitEarly(unsigned int _splitPos, unsigned int _levelIdx, unsigned int _predIdx, unsigned int _bufIdx, unsigned int _setIdx) {
-  splitPos = _splitPos;
-  levelIdx = _levelIdx;
-  predIdx = _predIdx;
-  bufIdx = _bufIdx;
-  setIdx = _setIdx;
-}
-
-
-/**
-   @brief Initializes field values known only following restaging.
-
-   @return void.
- */
-void SplitCoord::InitLate(const Bottom *bottom, const IndexLevel &index) {
-  unsigned int idxCount;
-  preBias = index.SplitFields(levelIdx, idxStart, idxCount, sCount, sum);
-  denseCount = bottom->AdjustDense(levelIdx, predIdx, idxStart, idxCount);
-
-  // Singletons may arise from bagging or persist from initialization:
-  if (idxCount == 0)
-    bottom->SetRunCount(levelIdx, predIdx, 1);
-  idxEnd = idxStart + idxCount - 1; // May overflow if singleton:  invalid.
-}
-
- 
 void SPReg::Split(const IndexLevel &index) {
+  ScheduleSplits(index);
+
   // Guards cast to int for OpenMP 2.0 back-compatibility.
   int splitPos;
 #pragma omp parallel default(shared) private(splitPos)
@@ -508,6 +537,8 @@ void SPReg::Split(const IndexLevel &index) {
 
 
 void SPCtg::Split(const IndexLevel &index) {
+  ScheduleSplits(index);
+  
   // Guards cast to int for OpenMP 2.0 back-compatibility.
   int splitPos;
 #pragma omp parallel default(shared) private(splitPos)
@@ -525,13 +556,6 @@ void SPCtg::Split(const IndexLevel &index) {
    @brief  Regression splitting based on type:  numeric or factor.
  */
 void SplitCoord::Split(const SPReg *spReg, const Bottom *bottom, const SamplePred *samplePred, const IndexLevel &index) {
-  InitLate(bottom, index);
-
-  // Bagging or restaging may precipitate new singletons.
-  //
-  if (bottom->Singleton(levelIdx, predIdx))
-    return;
-
   if (spReg->IsFactor(predIdx)) {
     SplitFac(spReg, bottom, samplePred->PredBase(predIdx, bufIdx));
   }
@@ -545,13 +569,6 @@ void SplitCoord::Split(const SPReg *spReg, const Bottom *bottom, const SamplePre
    @brief Categorical splitting based on type:  numeric or factor.
  */
 void SplitCoord::Split(SPCtg *spCtg, const Bottom *bottom, const SamplePred *samplePred, const IndexLevel &index) {
-  InitLate(bottom, index);
-  
-  // Bagging or restaging may precipitate new singletons.
-  //
-  if (bottom->Singleton(levelIdx, predIdx))
-    return;
-
   if (spCtg->IsFactor(predIdx)) {
     SplitFac(spCtg, bottom, samplePred->PredBase(predIdx, bufIdx));
   }
@@ -584,27 +601,23 @@ void SplitCoord::SplitNum(SPCtg *spCtg, const Bottom *bottom, const SPNode spn[]
 
 void SplitCoord::SplitFac(const SPReg *spReg, const Bottom *bottom, const SPNode spn[]) {
   NuxLH nux;
-  unsigned int runCount;
-  if (SplitFac(spReg, spn, runCount, nux)) {
+  if (SplitFac(spReg, spn, nux)) {
     bottom->SSWrite(levelIdx, predIdx, setIdx, bufIdx, nux);
   }
-  bottom->SetRunCount(levelIdx, predIdx, runCount);
 }
 
 
 void SplitCoord::SplitFac(const SPCtg *spCtg, const Bottom *bottom, const SPNode spn[]) {
   NuxLH nux;
-  unsigned int runCount;
-  if (SplitFac(spCtg, spn, runCount, nux)) {
+  if (SplitFac(spCtg, spn, nux)) {
     bottom->SSWrite(levelIdx, predIdx, setIdx, bufIdx, nux);
   }
-  bottom->SetRunCount(levelIdx, predIdx, runCount);
 }
 
 
-bool SplitCoord::SplitFac(const SPCtg *spCtg, const SPNode spn[], unsigned int &runCount, NuxLH &nux) {
+bool SplitCoord::SplitFac(const SPCtg *spCtg, const SPNode spn[], NuxLH &nux) {
   RunSet *runSet = spCtg->RSet(setIdx);
-  runCount = RunsCtg(spCtg, runSet, spn);
+  RunsCtg(spCtg, runSet, spn);
 
   if (spCtg->CtgWidth() == 2) {
     return SplitBinary(spCtg, runSet, nux);
@@ -623,15 +636,13 @@ bool SplitCoord::SplitFac(const SPCtg *spCtg, const SPNode spn[], unsigned int &
 /**
    @brief Weighted-variance splitting method.
 
-   @param runCount outputs recently-updated run count.
-
    @param nux outputs split nucleus.
 
    @return true iff pair splits.
  */
-bool SplitCoord::SplitFac(const SPReg *spReg, const SPNode spn[], unsigned int &runCount, NuxLH &nux) {
+bool SplitCoord::SplitFac(const SPReg *spReg, const SPNode spn[], NuxLH &nux) {
   RunSet *runSet = spReg->RSet(setIdx);
-  runCount = RunsReg(runSet, spn, spReg->DenseRank(predIdx));
+  RunsReg(runSet, spn, spReg->DenseRank(predIdx));
   runSet->HeapMean();
   runSet->DePop();
 
@@ -651,10 +662,10 @@ bool SplitCoord::SplitFac(const SPReg *spReg, const SPNode spn[], unsigned int &
 bool SplitCoord::SplitNum(const SPReg *spReg, const SPNode spn[], NuxLH &nux) {
   int monoMode = spReg->MonoMode(splitPos, predIdx);
   if (monoMode != 0) {
-    return denseCount > 0 ? SplitNumDenseMono(monoMode > 0, spn, spReg, nux) : SplitNumMono(monoMode > 0, spn, nux);
+    return implicit > 0 ? SplitNumDenseMono(monoMode > 0, spn, spReg, nux) : SplitNumMono(monoMode > 0, spn, nux);
   }
   else {
-    return denseCount > 0 ? SplitNumDense(spn, spReg, nux) : SplitNum(spn, nux);
+    return implicit > 0 ? SplitNumDense(spn, spReg, nux) : SplitNum(spn, nux);
   }
 
 }
@@ -704,7 +715,7 @@ bool SplitCoord::SplitNum(const SPNode spn[], NuxLH &nux) {
 
 
 /**
-   @brief Experimental.
+   @brief Experimental.  Needs refactoring.
 
    @return void.
 */
@@ -728,7 +739,7 @@ bool SplitCoord::SplitNumDense(const SPNode spn[], const SPReg *spReg, NuxLH &nu
   else {
     spn[idxEnd].RegFields(ySum, rkRight, sampleCount);
     idxNext = idxEnd - 1;
-    idxFinal = denseLeft ? idxStart : denseCut + 1;
+    idxFinal = denseLeft ? idxStart : denseCut;
   }
   double sumR = ySum;
   unsigned int sCountL = sCount - sampleCount;
@@ -769,7 +780,7 @@ bool SplitCoord::SplitNumDense(const SPNode spn[], const SPReg *spReg, NuxLH &nu
       maxInfo = idxGini;
     }
   
-    if (!denseLeft) {  // Walks remaining indices, if any, with rank below dense.
+    if (!denseLeft) { // Walks remaining indices, if any, with rank below dense.
       sCountL -= sCountDense;
       sumR += sumDense;
       rkRight = denseRank;
@@ -794,7 +805,7 @@ bool SplitCoord::SplitNumDense(const SPNode spn[], const SPReg *spReg, NuxLH &nu
   }
 
   if (maxInfo > preBias) {
-    unsigned int lhDense = rankLH >= denseRank ? denseCount : 0;
+    unsigned int lhDense = rankLH >= denseRank ? implicit : 0;
     unsigned int lhIdxTot = rhInf - idxStart + lhDense;
     nux.InitNum(idxStart, lhIdxTot, lhSampCt, maxInfo - preBias, rankLH, rankRH, lhDense);
     return true;
@@ -830,7 +841,7 @@ bool SplitCoord::SplitNumDenseMono(bool increasing, const SPNode spn[], const SP
   else {
     spn[idxEnd].RegFields(ySum, rkRight, sampleCount);
     idxNext = idxEnd - 1;
-    idxFinal = denseLeft ? idxStart : denseCut + 1;
+    idxFinal = denseLeft ? idxStart : denseCut;
   }
   double sumR = ySum;
   unsigned int sCountL = sCount - sampleCount;
@@ -902,7 +913,7 @@ bool SplitCoord::SplitNumDenseMono(bool increasing, const SPNode spn[], const SP
   }
 
   if (maxInfo > preBias) {
-    unsigned int lhDense = rankLH >= denseRank ? denseCount : 0;
+    unsigned int lhDense = rankLH >= denseRank ? implicit : 0;
     unsigned int lhIdxTot = rhInf - idxStart + lhDense;
     nux.InitNum(idxStart, lhIdxTot, lhSampCt, maxInfo - preBias, rankLH, rankRH, lhDense);
     return true;
@@ -970,20 +981,17 @@ bool SplitCoord::SplitNumMono(bool increasing, const SPNode spn[], NuxLH &nux) {
    @param sCount dense input the response sample count over the node and
    outputs the residual count.
 
-   @param denseCut output the supremum of indices to the left ot the
-   dense rank.
-
-   @return true iff left bound has rank less than dense value.
+   @return supremum of indices to the left ot the dense rank.
 */
 unsigned int SPReg::Residuals(const SPNode spn[], unsigned int idxStart, unsigned int idxEnd, unsigned int denseRank, unsigned int &denseLeft, unsigned int &denseRight, double &sumDense, unsigned int &sCountDense) const {
-  unsigned int denseCut = idxStart; // Defaults to lowest index.
+  unsigned int denseCut = idxEnd; // Defaults to highest index.
   double sumTot = 0.0;
   unsigned int sCountTot = 0;
   for (int idx = int(idxEnd); idx >= int(idxStart); idx--) {
     unsigned int sampleCount, rkThis;
     FltVal ySum;
     spn[idx].RegFields(ySum, rkThis, sampleCount);
-    denseCut = rkThis >= denseRank ? idx : denseCut;
+    denseCut = rkThis > denseRank ? idx : denseCut;
     sCountTot += sampleCount;
     sumTot += ySum;
   }
@@ -991,8 +999,8 @@ unsigned int SPReg::Residuals(const SPNode spn[], unsigned int idxStart, unsigne
   sCountDense -= sCountTot;
 
   // Dense blob is either left, right or neither.
-  denseRight = (denseCut == idxEnd && spn[idxEnd].Rank() < denseRank);  
-  denseLeft = (denseCut == idxStart && spn[idxStart].Rank() > denseRank);
+  denseRight = (denseCut == idxEnd && spn[denseCut].Rank() < denseRank);  
+  denseLeft = (denseCut == idxStart && spn[denseCut].Rank() > denseRank);
   
   return denseCut;
 }
@@ -1054,7 +1062,7 @@ void SPCtg::ApplyResiduals(unsigned int levelIdx, unsigned int predIdx, double &
 
 
 bool SplitCoord::SplitNum(SPCtg *spCtg, const SPNode spn[], NuxLH &nux) {
-  if (denseCount > 0) {
+  if (implicit > 0) {
     return NumCtgDense(spCtg, spn, nux);
   }
   else {
@@ -1176,7 +1184,7 @@ bool SplitCoord::NumCtgDense(SPCtg *spCtg, const SPNode spn[], NuxLH &nux) {
   }
 
   if (maxInfo > preBias) {
-    unsigned int lhDense = rankLH >= denseRank ? denseCount : 0;
+    unsigned int lhDense = rankLH >= denseRank ? implicit : 0;
     unsigned int lhIdxTot = rhInf - idxStart + lhDense;
     nux.InitNum(idxStart, lhIdxTot, lhSampCt, maxInfo - preBias, rankLH, rankRH, lhDense);
     return true;
@@ -1190,7 +1198,7 @@ bool SplitCoord::NumCtgDense(SPCtg *spCtg, const SPNode spn[], NuxLH &nux) {
 /**
    Regression runs always maintained by heap.
 */
-unsigned int SplitCoord::RunsReg(RunSet *runSet, const SPNode spn[], unsigned int denseRank) const {
+void SplitCoord::RunsReg(RunSet *runSet, const SPNode spn[], unsigned int denseRank) const {
   double sumHeap = 0.0;
   unsigned int sCountHeap = 0;
   unsigned int rkThis = spn[idxEnd].Rank();
@@ -1220,11 +1228,9 @@ unsigned int SplitCoord::RunsReg(RunSet *runSet, const SPNode spn[], unsigned in
   // Flushes the remaining run.  Also flushes the implicit run, if dense.
   //
   runSet->Write(rkThis, sCountHeap, sumHeap, frEnd - idxStart + 1, idxStart);
-  if (denseCount > 0) {
-    runSet->WriteImplicit(denseRank, sCount, sum, denseCount);
+  if (implicit > 0) {
+    runSet->WriteImplicit(denseRank, sCount, sum, implicit);
   }
-
-  return runSet->RunCount();
 }
 
 
@@ -1272,7 +1278,7 @@ bool SplitCoord::HeapSplit(RunSet *runSet, NuxLH &nux) const {
    when run count has been estimated to be wide:
 
 */
-unsigned int SplitCoord::RunsCtg(const SPCtg *spCtg, RunSet *runSet, const SPNode spn[]) const {
+void SplitCoord::RunsCtg(const SPCtg *spCtg, RunSet *runSet, const SPNode spn[]) const {
   double sumLoc = 0.0;
   unsigned int sCountLoc = 0;
   unsigned int rkThis = spn[idxEnd].Rank();
@@ -1302,11 +1308,9 @@ unsigned int SplitCoord::RunsCtg(const SPCtg *spCtg, RunSet *runSet, const SPNod
   
   // Flushes remaining run.
   runSet->Write(rkThis, sCountLoc, sumLoc, frEnd - idxStart + 1, idxStart);
-  if (denseCount > 0) {
-    runSet->WriteImplicit(spCtg->DenseRank(predIdx), sCount, sum, denseCount, spCtg->ColumnSums(levelIdx));
+  if (implicit > 0) {
+    runSet->WriteImplicit(spCtg->DenseRank(predIdx), sCount, sum, implicit, spCtg->ColumnSums(levelIdx));
   }
-
-  return runSet->RunCount();
 }
 
 
